local _ARTG = _Avyr_Raid_Tools_ADDON_Globals
local Widgets = _Avyr_Global_ADDON_Elements_Lib:Initialize("AvyrRaidTools","Widgets")
local Loc = _Avyr_Global_ADDON_Elements_Lib:Initialize("AvyrRaidTools","Localisation")
local BorderSize, Padding = Widgets.BorderSize, Widgets.Padding;
local insert,wipe = table.insert,table.wipe

_ARTG.DataFuncs = {}

local Data = _ARTG.DataFuncs


----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
-----------------------------------------------------------------------------------------CREDITS----------------------------------------------------------------------------------------
----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

Data.Credits = {
	"Avyr",
	"Parantava",
	"Flipz",
	"Nanuk",
	"Saphi",
	"Tyra"
}

----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
----------------------------------------------------------------------------------------CHANGELOG---------------------------------------------------------------------------------------
----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

--[[
Information display load conditions bug
global alpha
move editor into options menu
annoucer
sanctum premades
tutorial
test note
]]

Data.Changelog = {
	{"Version 2.08",
		{			
			"Added Sanction of Domination PTR notes. (Updates to come once bosses are live)",
			"Added note quick select for Sanction of Domination bosses.",
		}
	},
	{"Version 2.07",
		{			
			"Fix a load condition bug that would load the information display whenever any element was set to load in a raid instance.",
			"Added credits.",
		}
	},
	{"Version 2.06",
		{
			"Altered default settings for load conditions again.",
			"Changed the way displays are loaded (this should fix the bugs of elements scaling when hiding others).",
			"A display anchored to another can now be dragged changing to position of the parent display.",
			"Moved General keyword settings from Note Display/Keywords/General to Note Display/Settings these should make them easier to find and having a separate tab for 3 items (now 5 see below) made little sense.",
			"Added setting to change the font color of entries containing keywords.",
			"Added setting to hide entries that don't contain keywords."
		}
	},
	{"Version 2.05",
		{
			"Altered default settings (load conditions).",
			"Fixed bug that would result in black boxes for some ui elements in the backdrop of the options menu."
		}
	},
	{"Version 2.04",
		{
			"Removed note length restrictions for ingame sending of notes.",
			"Increased cooldown favourites from 40 to 100."
		}
	},
	{"Version 2.03.d",
		{
			"Verified Keal'thas, Xy mox, Darkvein and Council premade notes.",
			"Users can now manully scroll through notes during combat (this will not disable auto scroll).",
		}
	},
	{"Version 2.02",
		{
			"Slowly adding Castle Nathria boss templates (Shriekwing, Huntsman and Destroyer done).",
			"Added Castle Nathria bosses to mouseover note select list.",
		}
	},
	{"Version 2.01",
		{
			"Shadowlands beta templates.",
		}
	},
	{"Version 2.00",
		{
			"Shadowlands pre patch bug fixes.",
		}
	},
	{"Version 1.05.b",
		{
			"Load conditions bug fix.",
			"Improved German localisation.",
		}
	},
	{"Version 1.05",
		{
			"Increased scale limit on UI components to 4x (from 2x)",
			"Dropdown fields/button with dropdown menus now have a arrow for better indication that the field in question is a dropdown field.",
			"Added options for the note display popup (including only showing the popup when pressing a set modifier key (shift, alt or ctrl etc)).",
			"Fundamental rework of load conditions and the load conditions UI to extent functionality.",
			"Options menu simplification (eliminated the generel options sub category and consolidated Generel>Settings, General>Components and General>Overview into General settings)"
		}
	},
	{"Version 1.04.b",
		{
			"Keyword cooldowns will now be prioritized above keywords.",
			"Minimap icon improved to better work with square minimaps.",
			"Fixed a bug that incorrectly saved the position of some UI elements.",
			"Changed the name of note templates to premade notes."
		}
	},
	{"Version 1.04",
		{
			"Ra-den, Carapace of N'Zoth and N'Zoth templates added.",
			"Skitra template reworked (due to balance changes).",
			"Added changelog"
		}
	},
	{"Version 1.03",
		{
			"Sounds registered by SharedMediaLib can now be used as keyword sounds.",
			"Added send button to note quick select window."
		}
	},
}

----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------General functions-----------------------------------------------------------------------------------
----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
Data.cloneTable = function(table)
	if type(table) == "table" then 
		local clone = {}
		
		for i,v in pairs(table) do 
			if type(v) == "table" then 
				clone[i] = Data.cloneTable(v)
			else 
				clone[i] = v
			end 
		end 

		return clone
	else 
		return table
	end 
end

----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
-----------------------------------------------------------------------------Expansions, Raid and Boss Data-----------------------------------------------------------------------------
----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
local DefaultBossImage = "Interface\\EncounterJournal\\UI-EJ-BOSS-Default"
local expansionsData = {
	--Names will be overwritten with localised names 
	[1] = {
		Name = "Classic",
		Image = {"Interface\\LFGFRAME\\UI-LFG-BACKGROUND-AQRUINS",0,1,0.3,0.7},
	},
	[2] = {
		Name = "TBC",
		Image = {"Interface\\LFGFRAME\\UI-LFG-BACKGROUND-HELLFIRECITADEL",0,1,0.2,0.6},
	},
	[3] = {
		Name = "WotLK",
		Image = {"Interface\\LFGFRAME\\UI-LFG-BACKGROUND-ICECROWNCITADEL",0,1,0.1,0.5},
	},
	[4] = {
		Name = "Cata",
		Image = {"Interface\\LFGFRAME\\UI-LFG-BACKGROUND-FallofDeathwing",0,1,0.3,0.7},
	},
	[5] = {
		Name = "Panda",
		Image = {"Interface\\LFGFRAME\\UI-LFG-BACKGROUND-TempleoftheJadeSerpent",0,1,0.2,0.6},
	},
	[6] = {
		Name = "WoD",
		Image = {"Interface\\LFGFRAME\\UI-LFG-BACKGROUND-Draenor",0,1,0.3,0.7},
	},
	[7] = {
		Name = "Legion",
		Image = {"Interface\\LFGFRAME\\UI-LFG-BACKGROUND-Antorus",0,1,0.3,0.7},
	},
	[8] = {
		Name = "BfA",
		Image = {"Interface\\LFGFRAME\\UI-LFG-BACKGROUND-Ataldazar",0,1,0.3,0.7},
	},
	[9] = {
		Name = "SL",
		Image = {3850569,0.05,0.65,0.15,0.45},
	},
}
local raidsByExpansion = {}
local encounterInfoByID = {}
local bossByRaids = {}
local bossTemplates = {}
local bossMobIDLoadInfo = {
	--darmac (debugging)
	["76865"] = {1694,457,6},
	
	
	--wrathion
	["156818"] = {2329,1180,8},
	--maut
	["156523"] = {2327,1180,8},
	--The Prophet Skitra
	["157238"] = {2334,1180,8},
	--Dark Inquisitor Xanesh
	["156575"] = {2328,1180,8},
	--The Hivemind  
	["157253"] = {2333,1180,8},
	["157254"] = {2333,1180,8},
	--Shad'har the Insatiable
	["157231"] = {2335,1180,8},
	--Drest'agath
	["157602"] = {2343,1180,8},
	--Vexiona
	["157354"] = {2336,1180,8},
	--Ra-den the Despoiled
	["156866"] = {2331,1180,8},
	--Il'gynoth, Corruption Reborn
	["158328"] = {2345,1180,8},
	--Carapace of N'Zoth
	["157439"] = {2337,1180,8},
	--N'Zoth, the Corruptor
	["158041"] = {2344,1180,8},
	
	["164406"] = {2398, 1190, 9}, --Shriekwing
	["165066"] = {2418, 1190, 9}, --Huntsman Altimor
	["24664"] = {2402, 1190, 9}, --Sun Kings Salvation
	["168973"] = {2402, 1190, 9}, --Sun Kings Salvation (big mob in front of platform)
	["166644"] = {2405, 1190, 9}, --Artificer Xy mox
	["164261"] = {2383, 1190, 9}, --Hungering Destroyer
	["167517"] = {2406, 1190, 9}, --Lady Inerva Darkvein	
	["166971"] = {2412, 1190, 9}, --The Council of blood (niklaus)
	["166969"] = {2412, 1190, 9}, --The Council of blood (frieda)
	["166970"] = {2412, 1190, 9}, --The Council of blood (stavros)
	["164407"] = {2399, 1190, 9}, --Sludgefist	
	["165318"] = {2417, 1190, 9}, --Stone legion generals (Kaal)
	["170323"] = {2417, 1190, 9}, --Stone legion generals (Grashaal)
	["168938"] = {2407, 1190, 9}, --Sire Denathrius	
	
	["152253"] = {2423, 1193, 9}, --The Tarragrue	
	["180018"] = {2433, 1193, 9}, --Eye of the jailer	
	["178738"] = {2429, 1193, 9}, --The Nine   kyra
	["178736"] = {2429, 1193, 9}, --The Nine   signe
	["178684"] = {2429, 1193, 9}, --The Nine   Skyja 	
	["175727"] = {2434, 1193, 9}, --Soulrender Dormazain	
	["175729"] = {2432, 1193, 9}, --Remnant of Ner'zhul	
	["176523"] = {2430, 1193, 9}, --Painsmith Raznal	
	["175731"] = {2436, 1193, 9}, --Guardian of the First Ones
	["179390"] = {2431, 1193, 9}, --Fatescribe Roh-Kalo
	["175559"] = {2422, 1193, 9}, --Kel'Thuzad
	["180828"] = {2435, 1193, 9}, --Sylvanas Windrunner
}
local bossLocationInfo = {
	
}

GameTooltip:HookScript("OnTooltipSetUnit",function(self)
	local UnitName,UnitID = self:GetUnit()
	if UnitID and _ARTG.SV.GetVariable("ND_PP_Enable") then 
		local GUID = UnitGUID(UnitID)
		local _,_,_,_,_,mobID = strsplit("-",GUID)
		local bossInfo = bossMobIDLoadInfo[mobID]
		if bossInfo and _ARTG.SV.GetVariable("ND_PP_Always") then 
			return _ARTG.NoteDisplay.CallLoadNotePopup("show.bossNotes",unpack(bossInfo))
		elseif bossInfo and (_ARTG.SV.GetVariable("ND_PP_CTRL") or _ARTG.SV.GetVariable("ND_PP_ALT") or _ARTG.SV.GetVariable("ND_PP_SHIFT")) then 
			local gameTooltipLine = "ART: " .. Loc:Translation("SHOW_NOTES_PRESS")..": "
			if _ARTG.SV.GetVariable("ND_PP_CTRL") and _ARTG.SV.GetVariable("ND_PP_ALT") and _ARTG.SV.GetVariable("ND_PP_SHIFT") then 
				gameTooltipLine = gameTooltipLine .. Loc:Translation("CTRL") .. ", " .. Loc:Translation("ALT") .. " " .. Loc:Translation("OR") .. " " .. Loc:Translation("SHIFT")
			elseif _ARTG.SV.GetVariable("ND_PP_CTRL") and _ARTG.SV.GetVariable("ND_PP_ALT") then 
				gameTooltipLine = gameTooltipLine .. Loc:Translation("CTRL") .. " " .. Loc:Translation("OR") .. " " .. Loc:Translation("ALT")
			elseif _ARTG.SV.GetVariable("ND_PP_ALT") and _ARTG.SV.GetVariable("ND_PP_SHIFT") then 
				gameTooltipLine = gameTooltipLine .. Loc:Translation("ALT") .. " " .. Loc:Translation("OR") .. " " .. Loc:Translation("SHIFT")
			elseif _ARTG.SV.GetVariable("ND_PP_CTRL") and _ARTG.SV.GetVariable("ND_PP_SHIFT") then
				gameTooltipLine = gameTooltipLine .. Loc:Translation("CTRL") .. " " .. Loc:Translation("OR") .. " " .. Loc:Translation("SHIFT")			
			elseif _ARTG.SV.GetVariable("ND_PP_CTRL") then 
				gameTooltipLine = gameTooltipLine .. Loc:Translation("CTRL")
			elseif _ARTG.SV.GetVariable("ND_PP_ALT") then 
				gameTooltipLine = gameTooltipLine .. Loc:Translation("ALT")
			elseif _ARTG.SV.GetVariable("ND_PP_SHIFT") then
				gameTooltipLine = gameTooltipLine .. Loc:Translation("SHIFT")
			end
			GameTooltip:AddLine(gameTooltipLine)
			if (_ARTG.SV.GetVariable("ND_PP_CTRL") and IsControlKeyDown()) or 
				(_ARTG.SV.GetVariable("ND_PP_ALT") and IsAltKeyDown()) or 
				(_ARTG.SV.GetVariable("ND_PP_SHIFT") and IsShiftKeyDown()) then 
				return _ARTG.NoteDisplay.CallLoadNotePopup("show.bossNotes",unpack(bossInfo))
			end
		end
	end 
end)

Data.RequiresLoadPrompt = function(UnitID)
	if UnitID and _ARTG.SV.GetVariable("ND_PP_Enable") and not _ARTG.SV.GetVariable("ND_PP_Always") then 
		local GUID = UnitGUID(UnitID)
		local _,_,_,_,_,mobID = strsplit("-",GUID)
		local bossInfo = bossMobIDLoadInfo[mobID]
		if bossInfo and ((_ARTG.SV.GetVariable("ND_PP_CTRL") and IsControlKeyDown()) or 
			(_ARTG.SV.GetVariable("ND_PP_ALT") and IsAltKeyDown()) or 
			(_ARTG.SV.GetVariable("ND_PP_SHIFT") and IsShiftKeyDown())) then 
			return _ARTG.NoteDisplay.CallLoadNotePopup("show.bossNotes",unpack(bossInfo))
		end
	end 
end


local function scanExpansions(forceScan,expansionIndex)
	if type(expansionIndex) == "number" then 
		if expansionIndex > 0 and expansionIndex <= EJ_GetNumTiers() and expansionsData[expansionIndex] then
			if expansionsData[expansionIndex].Checked and not forceScan then 
				return expansionIndex
			else 
				local Name,Link = EJ_GetTierInfo(expansionIndex)
				if Name then 
					expansionsData[expansionIndex].Name = Name 
					return  expansionIndex
				end
			end
		end
	else
		for i,v in ipairs(expansionsData) do 
			if not v.Checked or forceScan then 
				local Name,Link = EJ_GetTierInfo(i)
				if Name then 
					v.Name = Name 
				end
			end 
		end 
	end
end

local function createExpansionRaidList(expansionIndex,targetRaid)
	EJ_SelectTier(expansionIndex)

	local foundIt = false

	raidsByExpansion[expansionIndex] = raidsByExpansion[expansionIndex] or {}
	table.wipe(raidsByExpansion[expansionIndex])

	for raid = 1,100 do 
		local instanceID, name, description, bgImage, buttonImage1, loreImage, buttonImage2, dungeonAreaMapID, link, shouldDisplayDifficulty = EJ_GetInstanceByIndex(raid,true)
		if instanceID then 
			insert(raidsByExpansion[expansionIndex],instanceID)
			if type(targetRaid) == "number" and targetRaid == instanceID then 
				foundIt = true 
			end
		else
			break
		end 					
	end 

	return foundIt
end

local function createBossList(raidID,targetBoss)
	EJ_SelectInstance(raidID)

	bossByRaids[raidID] = bossByRaids[raidID] or {}
	table.wipe(bossByRaids[raidID])

	local foundIt = false 
	for encounter = 1,100 do 
		local name, description, EJ_encounterID, rootSectionID, link, instanceID, encounterID = EJ_GetEncounterInfoByIndex(encounter,raidID)
		local _, _, _, _, bossImage = EJ_GetCreatureInfo(1, EJ_encounterID);
		
		if name and encounterID then 
			insert(bossByRaids[raidID],encounterID)
			encounterInfoByID[encounterID] = {EJ_encounterID = EJ_encounterID, imagePath = bossImage};
			if type(targetBoss) == "number" and encounterID == targetBoss then 
				foundIt = true 
			end
		else 
			if not name then --Kara (TBC) servant quarter boss catch
				break 
			end 
		end 
	end 
	
	return foundIt
end

local function scanExpansionsForRaid(raidID)
	local foundIt = false
	for i = 1,EJ_GetNumTiers() do 
		if raidsByExpansion[i] then 
			if tContains(raidsByExpansion[i],raidID) then 
				foundIt = true 
			end
		else
			if createExpansionRaidList(i,raidID) then 
				foundIt = true 
			end
		end 
		if foundIt then 
			return true 
		end 
	end
	return false 
end

local function ScanRaidsForBoss(bossID)
	for i = 1,EJ_GetNumTiers() do 
		if not raidsByExpansion[i] then 
			createExpansionRaidList(i,raidID)
		end 

		for _,raidID in ipairs(raidsByExpansion[i]) do 
			if createBossList(raidID,bossID) then 
				return true 
			end 
		end
	end
	return false 
end 

local function scanRaids(forceScan,raidID,expansionIndex) 
	if type(raidID) == "number" then 
		if bossByRaids[raidID] then --force scan integration
			return true 
		end

		local raidFound = false 
		if _ARTG.DataFuncs.GetRaidInfo("exp.find",expansionIndex) then 
			if not raidsByExpansion[expansionIndex] or forceScan then 
				raidFound = createExpansionRaidList(expansionIndex,raidID)
			end
	
			if raidFound then 
				return true 
			end
		end 

		return scanExpansionsForRaid(raidID)
	elseif not raidsByExpansion[expansionIndex] or forceScan then 
		createExpansionRaidList(expansionIndex)
	end 

	return false 
end

local function scanBosses(forceScan,bossID,raidID)
	if type(bossID) == "number" then 
		if encounterInfoByID[bossID] then 
			return true 
		end
		
		local bossFound = false
		if type(raidID) == "number" and EJ_GetInstanceInfo(raidID) then 
			 bossFound = createBossList(raidID,bossID)
		end 

		if bossFound then 
			return true 
		end

		return ScanRaidsForBoss(bossID)
	end
	return false 
end

Data.GetRaidInfo = function(Action,...)
	if Action == "none" then 

	elseif Action == "boss.find" then 
		local bossID,raidID = ...

		return scanBosses(false,bossID,raidID)	
	elseif Action == "boss.list" then 
		local raidID,expansionIndex = ...

		local foundIt = scanBosses(false)

		if type(raidID) == "number" then 
			createBossList(raidID)

			return bossByRaids[raidID]
		end

		return {}
	elseif Action:find("boss.data") then
		local bossID,raidID,expansionIndex = ...

		local name = "Unknown boss"
		local imagePath = DefaultBossImage

		if type(bossID) == "number" and scanBosses(false,bossID,raidID,expansionIndex) then 
			name = EJ_GetEncounterInfo(encounterInfoByID[bossID]["EJ_encounterID"])
			imagePath = encounterInfoByID[bossID]["imagePath"]
		end

		if Action:find(".name") then 
			return name
		elseif Action:find(".image") then 
			return imagePath
		else 
			return name, imagePath
		end
	elseif Action == "raid.find" then 
		local raidID,expansionIndex = ...

		return scanRaids(false,raidID,expansionIndex)
	elseif Action == "raid.list" then 
		local expansionIndex = ...
		
		if expansionIndex then 
			scanRaids(false,nil,expansionIndex)	
			if raidsByExpansion[expansionIndex] then 
				return raidsByExpansion[expansionIndex]
			else
				return bossByRaids
			end 
			return bossByRaids
		end

		return {}
	elseif Action:find("raid.data") then 
		local raidID = ...
		local name = "Unknown raid"
		local imagePath = DefaultBossImage

		if type(raidID) == "number" then 
			local RaidName,_,_,Icon = EJ_GetInstanceInfo(raidID) 
			if RaidName then 
				name = RaidName
			end 
			if Icon then 
				imagePath = Icon
			end
		end

		if Action:find(".name") then 
			return name
		elseif Action:find(".image") then 
			return imagePath
		else 
			return name,imagePath
		end 
	elseif Action == "exp.find" then 
		local expansionIndex = ...
		if type(expansionIndex) == "number" and expansionIndex > 0 and expansionIndex <= EJ_GetNumTiers() and expansionsData[expansionIndex] then 
			return true 
		else 
			return false 
		end 
	elseif Action == "exp.list" then
		scanExpansions()

		return expansionsData
	elseif Action:find("exp.data") then 
		--exp.data.image
		--exp.data.name
		local expansionIndex = ...
		local Index = nil

		Index = scanExpansions(false,expansionIndex)
		
		if Index then 
			if Action:find(".image") then 
				return expansionsData[Index].Image[1],expansionsData[Index].Image[2],expansionsData[Index].Image[3],expansionsData[Index].Image[4],expansionsData[Index].Image[5]
			else --".name"
				return expansionsData[Index].Name
			end 
		end
	end
end

----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
-----------------------------------------------------------------------------------------Notes------------------------------------------------------------------------------------------
----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
local selectedNote = {}
Data.NoteData = function(Action,...)
	if Action == "current.set" then 
		local NoteTable,expansionIndex,raidID,bossID = ...

		local ClonedNote = Data.cloneTable(NoteTable)
		ClonedNote.Expansion = expansionIndex
		ClonedNote.Raid = raidID
		ClonedNote.Boss = bossID
		_ARTG.SV.SaveVariable("CurrentNote",ClonedNote)
	elseif Action == "current.get" then 
		return _ARTG.SV.GetVariable("CurrentNote")
	elseif Action == "current.set.selected" then 
		_ARTG.SV.SaveVariable("CurrentNote",Data.cloneTable(selectedNote))
	elseif Action == "selected.set" then 
		local NoteTable,expansionIndex,raidID,bossID = ...
		
		local ClonedNote = Data.cloneTable(NoteTable)
		ClonedNote.Expansion = expansionIndex
		ClonedNote.Raid = raidID
		ClonedNote.Boss = bossID

		wipe(selectedNote)

		selectedNote = ClonedNote
	elseif Action == "selected.get" then 
		return Data.cloneTable(selectedNote)
	end
end

----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
-------------------------------------------------------------------------------------Cooldowns Data-------------------------------------------------------------------------------------
----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------



----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------Inspect Data--------------------------------------------------------------------------------------
----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------






----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------SOUND Data---------------------------------------------------------------------------------------
----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
local soundTable = {
	{Loc:Translation("WORLD_QUEST_START_SOUND"),
	Loc:Translation("RAID_BOSS_DEFEATED_SOUND"),
	Loc:Translation("SCENARIO_ENDING_SOUND"),
	Loc:Translation("ALARM_CLOCK_1_SOUND"),
	Loc:Translation("ALARM_CLOCK_2_SOUND"),
	Loc:Translation("ALARM_CLOCK_3_SOUND"),
	Loc:Translation("RAID_WARNING_SOUND"),
	Loc:Translation("PING_SOUND"),
	Loc:Translation("MONEY_SOUND"),
	Loc:Translation("OPEN_BAGS_SOUND"),
	Loc:Translation("READY_CHECK_SOUND"),},
	{73275,50111,31754,18871,12867,12889,8959,3175,120,862,8960},
}

local ShareMediaLib = LibStub("LibSharedMedia-3.0")
local sharedMediaLibHashTable = ShareMediaLib:HashTable(ShareMediaLib.MediaType.SOUND)

ART_global_testFunc = function()
	local l = ShareMediaLib:HashTable(ShareMediaLib.MediaType.SOUND)


	for i,v in pairs(l) do 
		print(i,v)
	end
	
	print(ShareMediaLib:Fetch(ShareMediaLib.MediaType.SOUND,"None"))
end

--/run ART_global_testFunc()
Data.Sounds = function(Action,...)
	if Action == "sound.play" then 
		local sound = ...
		if type(sound) == "number" then 
			PlaySound(sound,"Master")
		elseif type(sound) == "string" and sharedMediaLibHashTable[sound] then 
			PlaySoundFile(sharedMediaLibHashTable[sound],"Master")
		end
	elseif Action == "list.get" then 
		local soundSelect = Data.cloneTable(soundTable)
		for i,v in pairs(sharedMediaLibHashTable) do 
			insert(soundSelect[1],i)
			insert(soundSelect[2],i)
		end
		return soundSelect
	elseif Action:find("sound.get") then 
		local soundIndex
		if Action:find(".byIndex") then 
			soundIndex,soundName = ...
			if soundName and sharedMediaLibHashTable[soundName] then 
				return soundName
			end
		elseif Action:find(".byName") then 
			local soundName = ...
			if soundName then 
				if sharedMediaLibHashTable[soundName] then 
					return soundName
				end
				for i,v in pairs(soundTable[1]) do 
					if soundTable[1][i] == soundName then 
						soundIndex = i
					end 
				end
			end 	
		elseif Action:find(".byId") then 
			local soundId = ...
			if soundId then 
				if type(soundId) == "number" then
					for i,v in pairs(soundTable[1]) do 
						if soundTable[2][i] == soundId then 
							soundIndex = i
						end 
					end
				elseif type(soundId) == "string" and sharedMediaLibHashTable[soundId] then 
					return soundId
				end
			end 	
		end

		if soundIndex then 
			if type(soundIndex) == "number" and soundTable[1][soundIndex] then 
				if Action:find(".name") then 
					return soundTable[1][soundIndex]
				elseif Action:find(".id") then 
					return soundTable[2][soundIndex]
				end
			elseif type(soundIndex) == "string" and sharedMediaLibHashTable[soundIndex] then 
				return soundIndex --shared media id is the name
			end
		end

		return soundIndex
	end 
end

----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
-------------------------------------------------------------------------------------Load Conditions------------------------------------------------------------------------------------
----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

local listOfLoadConditions = {
	"ALWAYS",

	"C_NOTE",

	"HEALER",
	"DAMAGER",
	"TANK",

	"RAID_GROUP",
	"PARTY",
	"SOLO",
	
	"OPEN_WORLD",
	"RAID_INSTANCE",
	"INSTANCE",
	"KEYSTONE",
}

Data.LoadConditions = function(Action,...)
    if Action == "display.check" then 
        local displayType = ...
        if type(displayType) == "string" then
			local SpecRole = GetSpecializationRole(GetSpecialization() or 1) or "NONE"
			local CurrentNote = _ARTG.SV.GetVariable("CurrentNote")["Name"]
			local partyStatus = "SOLO"
			if UnitGUID("raid1") then
				partyStatus = "RAID_GROUP"
			elseif UnitGUID("party1") then 
				partyStatus = "PARTY"
			end
			local InstanceType = "OPEN_WORLD"
			if C_ChallengeMode.IsChallengeModeActive() then 
				InstanceType = "KEYSTONE"
			elseif select(2,IsInInstance()) == "party" then 
				InstanceType = "INSTANCE"
			elseif select(2,IsInInstance()) == "raid" then 
				InstanceType = "RAID_INSTANCE"
			end
            for i,v in ipairs(AvyrRaidTools_DB.LoadConditions) do 
				local displayTypes,loadConditions = strsplit("|",v)
                if displayTypes:find(displayType) then 
					local loadConditionsTable = {strsplit(":",loadConditions or "")}
					--local allConditionsTrue = false --with no conditions it will always be false 
					if (loadConditionsTable[1] and loadConditionsTable[1] == "1") --always load
						or ((loadConditionsTable[2] and (loadConditionsTable[2] ~= "1" or CurrentNote))
							and (loadConditionsTable[3] and (loadConditionsTable[3] == "" or loadConditionsTable[3] == SpecRole or (loadConditionsTable[3]:find("!") and loadConditionsTable[3] ~= "!"..SpecRole)))
							and (loadConditionsTable[4] and (loadConditionsTable[4] == "" or loadConditionsTable[4] == InstanceType or (loadConditionsTable[4]:find("!") and loadConditionsTable[4] ~= "!"..InstanceType)))
							and (loadConditionsTable[5] and (loadConditionsTable[5] == "" or loadConditionsTable[5] == partyStatus or (loadConditionsTable[5]:find("!") and loadConditionsTable[5] ~= "!"..partyStatus)))
						)
					then 	
						return true 
					end

					--[[
					for i,v in ipairs(loadConditionsTable) do 
						if i == 1 then 
							allConditionsTrue = true 
						end
						--print(v,SpecRole,v == SpecRole,v ~= "!"..SpecRole)
						local absv = v:gsub("!","")
						if v == "ALWAYS" then 
							return true --override load
						elseif not (
							(v == "C_NOTE" and CurrentNote) 
							or (v == "!C_NOTE" and not CurrentNote)
							or ((absv == "HEALER" or absv == "TANK" or absv == "DAMAGER") and (v == SpecRole or v ~= "!"..SpecRole)) --role check
							or ((absv == "SOLO" or absv == "RAID_GROUP" or absv == "PARTY") and (v == partyStatus or v ~= "!"..partyStatus)) --group status check
							or ((absv == "OPEN_WORLD" or absv == "KEYSTONE" or absv == "INSTANCE" or absv == "RAID_INSTANCE") and (v == InstanceType or v ~= "!"..InstanceType)) --instance type check
						) then 
							allConditionsTrue = false 
						end
					end
					if allConditionsTrue then 
						return true 
					end]]
                end
            end
			return false
        end 
    elseif Action == "table.unpack" then 
        local loadConditionString = ...
        local displayTypes,loadConditions = strsplit("|",loadConditionString)

        local noteDisplayEnable = displayTypes:find("ND")
        local informationDisplayEnable = displayTypes:find("ID")
        local healerManaDisplayEnable = displayTypes:find("HMD")
		
		local AlwaysEnable,NoNoteDisable,Role,PartyStatus,InstanceType = false,false,"","",""
        local loadConditionsTable = {strsplit(":",(loadConditions or ""))}
		if loadConditionsTable[1] and loadConditionsTable[1] == "1" then 
			AlwaysEnable = true 
		end
		if loadConditionsTable[2] and loadConditionsTable[2] == "1" then 
			NoNoteDisable = true 
		end
		if loadConditionsTable[3] then 
			Role = loadConditionsTable[3]
		end
		if loadConditionsTable[4] then 
			InstanceType = loadConditionsTable[4]
		end
		if loadConditionsTable[5] then 
			PartyStatus = loadConditionsTable[5]
		end

        return noteDisplayEnable,informationDisplayEnable,healerManaDisplayEnable,AlwaysEnable,NoNoteDisable,Role,InstanceType,PartyStatus
	elseif Action == "entry.create" then 
		local noteDisplayEnable,informationDisplayEnable,healerManaDisplayEnable,AlwaysEnable,NoNoteDisable,Role,InstanceType,PartyStatus = ...
		local conditionsString = ""
		if noteDisplayEnable then 
			conditionsString = conditionsString .. "ND"
		end 
		if informationDisplayEnable then 
			if conditionsString ~= "" then 
				conditionsString = conditionsString .. ":"			
			end 
			conditionsString = conditionsString .. "ID"
		end
		if healerManaDisplayEnable then 
			if conditionsString ~= "" then 
				conditionsString = conditionsString .. ":"			
			end 
			conditionsString = conditionsString .. "HMD"
		end

		conditionsString = conditionsString .. "|"

		if AlwaysEnable then 
			conditionsString = conditionsString .. "1"
		else 
			conditionsString = conditionsString .. "0"
		end 

		if NoNoteDisable then 
			conditionsString = conditionsString .. ":1"
		else 
			conditionsString = conditionsString .. ":0"
		end 

		conditionsString = conditionsString .. ":" .. Role .. ":" .. InstanceType .. ":" .. PartyStatus
	
		return conditionsString
    end 
end


