local Widgets = _Avyr_Global_ADDON_Elements_Lib.WidgetsMixin
local pi, sin, cos, ipairs, wipe, select, insert, UIParent, CreateFrame, InCombatLockdown, GetCursorPosition, sqrt = math.pi, math.sin, math.cos, ipairs, table.wipe, select, table.insert, UIParent, CreateFrame, InCombatLockdown, GetCursorPosition, math.sqrt;
local pairs, type = pairs, type;
local BorderSize, Padding = Widgets.BorderSize, Widgets.Padding;
local _Avyr_GUI_Widgets_Tooltip = _Avyr_GUI_Widgets_Tooltip;

local function CornerRotationCalc(deg)
    local rads = deg / 180 * pi;
    local x = 0.5 + cos(rads) / sqrt(2)
    local y = 0.5 - sin(rads) / sqrt(2)
    return x, y
end

local function RotationPoint(deg)
    local deg = deg%360
    if deg <= 90 then 
        return 0
    elseif deg <= 180 then 
        return deg/90 - 1
    elseif deg <= 270 then 
        return 1
    else 
        return 4 - deg/90
    end 
end

local function RotationZoom(PointA,PointB,Zoom)
    if PointA ~= 1 and PointA ~= 0 then 
        if PointB == 1 then 
            PointB = 1 + Zoom
        elseif PointB == 0 then 
            PointB = -Zoom
        end 
    elseif PointB ~= 1 and PointB ~= 1 then 
        if PointA == 1 then 
            PointA = 1 + Zoom
        elseif PointA == 0 then 
            PointA = -Zoom
        end 
    end 
    return PointA,PointB
end 

local Frames = {}

local function Release(self)
    
    --[[
    for i, v in ipairs(self.LM) do
        if type(self[v]) == "table" and self[v].Release then
            self[v]:Release();
        end
        self[v] = nil
    end
    wipe(self.LM);
]]

	self:ClearPopup()

    self:Hide();

    --[[
        for i, v in ipairs(self.OnReleaseFuncs) do
            v(self)
        end
        wipe(self.OnReleaseFuncs)]]
    wipe(self.OnSizeChangedFuncs)
    self:SetThemedBackdrop("Clear")
    for i = 1, 20 do
        if self["Text" .. i] then
            self["Text" .. i]:Release()
            self["Text" .. i] = nil
        end
    end
    if self.Icon then
        self.Icon:Release()
        self.Icon = nil
    end
    self:SetFrameStrata("MEDIUM")
    self:SetFrameLevel(0)
    self:EnableMouse(false)
    self:SetDisable()
    self:SetScale(1)
	self:SetSize(0,0)
    self:SetClampedToScreen(false)
    self:SetScript("OnMouseWheel", nil)
    self:SetScript("OnEnter", nil)
    self:SetScript("OnLeave", nil)
    self:SetScript("OnMouseDown", nil)
    self:SetScript("OnMouseUp", nil)
    self:SetScript("OnUpdate", nil)
    self:SetScript("OnEvent", nil)
    self:UnregisterAllEvents()
    self:EnableMouse(false)
    self.OnMove = nil
    self.CheckCombatLock = nil
    self.PosParameters = nil
    self.MouseInteraction = nil
    self.EnterLeave = nil
    self.Borderless = nil;
    
    for i,v in pairs(self) do 
        if type(v) == "table" and v.Release then
            v:Release();
        end
        v = nil
    end 
end

local function ClearPopup(self)
    if self.ConnectedPopup then 
        self.ConnectedPopup:Release();
        self.ConnectedPopup = nil;
    end 
end

local function SetDisable(self, Value)
    if Value then
        self:SetAlpha(0.4)
    else
        self:SetAlpha(1)
    end
    self.Disable = Value
end

local tempDataStore = {}
local function SetText(self, ...)
--[[
        wipe(tempDataStore)
        local i = 1
        local CurrentItemIndex = 0
        local VarName,VarValue = select((2*i - 1),...)
        while type(VarName) == "string" do 
            if select((2*i - 1),...) == "Type" then
                insert(tempDataStore,{})
                CurrentItemIndex = CurrentItemIndex + 1
            end
            if tempDataStore[CurrentItemIndex] then 
                tempDataStore[CurrentItemIndex][select((2*i - 1),...)] = select(2*i,...)
            end
            i = i + 1
            VarName,VarValue = select((2*i - 1),...)
        end 


        --Frame:SetText("Text","THIS IS THE INPUT","ID",1,"HAlign","LEFT")
        --Frame:SetText("Clear")
]]


    local firstValue = ...
    if type(firstValue) == "table" then 
            for ri = 1, 20 do
                if self["Text" .. ri] then
                    self["Text" .. ri]:Release()
                    self["Text" .. ri] = nil
                else
                    break
                end
            end

            local DontResize = false
            local Height = 2*self.self.BorderSize
            for i,v in ipairs(...) do
                if not self["Text" .. i] then
                    self["Text" .. i] = self.self:FetchFontString(self)
                end
                DontResize = v.DontResize
                if i > 1 and not v.Overlapping then 
                    self["Text" .. i]:SetPoint("TOPLEFT",self["Text" .. i - 1],"BOTTOMLEFT", 0, -self.self.Padding)
                    self["Text" .. i]:SetPoint("TOPRIGHT",self["Text" .. i - 1],"BOTTOMRIGHT", 0, -self.self.Padding)    
                    Height = Height + self.self.Padding
                else 
                    self["Text" .. i]:SetPoint("TOPLEFT", self.self.BorderSize + self.self.Padding, -self.self.BorderSize - self.self.Padding)
                    self["Text" .. i]:SetPoint("TOPRIGHT", -self.self.BorderSize - self.self.Padding, -self.self.BorderSize - self.self.Padding)
                    Height = Height + 2*self.self.Padding
                end  
                self["Text" .. i]:SetText(v.Text or "")
                if type(v.r) == "string" then 
                    if v.r == "Highlight" then 
                        self.self:SetThemedColor(self["Text" .. i],"Highlight")
                    elseif v.r == "Normal" then 
                        self.self:SetThemedColor(self["Text" .. i],"Normal")
                    end 
                elseif v.r and v.g and v.b then
                    self["Text" .. i]:SetTextColor(v.r, v.g, v.b)
                end
                if v.FontSize or v.Flags then
                    local Font, _, Flags = self["Text" .. i]:GetFont()
                    self["Text" .. i]:SetFont(Font, (v.FontSize or 11), (v.Flags or Flags))
                    Height = Height + (v.FontSize or 11)
                else 
                    Height = Height + 11
                end
                if v.HAlign then
                    self["Text" .. i]:SetJustifyH(v.HAlign)
                end
                if v.VAlign then
                    self["Text" .. i]:SetJustifyV(v.VAlign)
                end
                self["Text" .. i]:SetWordWrap(v.Wrap)
            end
            if not DontResize then 
                self:Size(nil,Height)
            end 
    else 
        for i = 1, 20 do
            if not select((1 + 7 * (i - 1)), ...) then
                for ri = i, 20 do
                    if self["Text" .. ri] then
                        self["Text" .. ri]:Release()
                        self["Text" .. ri] = nil
                    else
                        break
                    end
                end
                break
            end
            local Text, FontSize, r, g, b, HAlign, VAlign = select((1 + 7 * (i - 1)), ...)
            if not self["Text" .. i] then
                self["Text" .. i] = self.self:FetchFontString(self)
                self["Text" .. i]:SetPoint("BOTTOMLEFT", self.self.BorderSize + self.self.Padding, self.self.BorderSize + self.self.Padding)
                self["Text" .. i]:SetPoint("TOPRIGHT", -self.self.BorderSize - self.self.Padding, -self.self.BorderSize - self.self.Padding)
            end
            self["Text" .. i]:SetText(Text or "")
            if type(r) == "string" then 
                if r == "Highlight" then 
                    self.self:SetThemedColor(self["Text" .. i],"Highlight")
                elseif r == "Normal" then 
                    self.self:SetThemedColor(self["Text" .. i],"Normal")
                end 
            elseif r and g and b then
                self["Text" .. i]:SetTextColor(r, g, b)
            end
            if FontSize then
                local Font, _, Flags = self["Text" .. i]:GetFont()
                self["Text" .. i]:SetFont(Font, FontSize, Flags)
            end
            if HAlign then
                self["Text" .. i]:SetJustifyH(HAlign)
            end
            if VAlign then
                self["Text" .. i]:SetJustifyV(VAlign)
            end
        end
    end 
end 

local tempIconData = {}
local function SetIcon(self, ...)
    --TexturePath, l, r, t, b, rC, gC, bC, BorderGap, Rotation, LockAspectRatio, Zoom
    --TexturePath, LeftCoord, RightCoord, TopCoord, BottomCoord, Red, Green, Blue, BorderGap, Rotation, LockAspectRatio, Zoom
    wipe(tempIconData)
    local i = 1
    local VarName,VarValue = select((2*i - 1),...)
    while type(VarName) == "string" do 
        if tempIconData then 
            tempIconData[VarName] = VarValue
        end
        i = i + 1
        VarName,VarValue = select((2*i - 1),...)
    end 

    if tempIconData["TexturePath"] then
        if not self.Icon then
            self.Icon = self.self:FetchTexture(self)
        end
        self.Icon:ClearAllPoints();
        local Gap = self.self.BorderSize
        if type(tempIconData["BorderGap"]) == "number" then
            Gap = tempIconData["BorderGap"]
        elseif tempIconData["BorderGap"] then 
            Gap = Gap + self.self.Padding
        end
        if tempIconData["LockAspectRatio"] then
            if type(tempIconData["LockAspectRatio"]) == "boolean" then
                self.Icon:SetPoint("CENTER")
                local MinSize = self:GetHeight() < self:GetWidth() and self:GetHeight() or self:GetWidth()
                self.Icon:SetSize(MinSize - 2 * Gap, MinSize - 2 * Gap)
            elseif tempIconData["LockAspectRatio"] == "LEFT" then
                self.Icon:SetPoint("BOTTOMLEFT", Gap, Gap)
                self.Icon:SetPoint("TOPRIGHT", self, "TOPLEFT", (self:GetHeight() - Gap), -Gap)
            elseif tempIconData["LockAspectRatio"] == "RIGHT" then
                self.Icon:SetPoint("BOTTOMRIGHT", -Gap, Gap)
                self.Icon:SetPoint("TOPLEFT", self, "TOPRIGHT", -(self:GetHeight() - Gap), -Gap)
            end
        else
            self.Icon:SetPoint("BOTTOMLEFT", Gap, Gap)
            self.Icon:SetPoint("TOPRIGHT", -Gap, -Gap)
        end
        self.Icon:SetTexture(tempIconData["TexturePath"])
        local TLx, TLy = 0,0
        local BLx, BLy = 0,1
        local TRx, TRy = 1,0
        local BRx, BRy = 1,1
        if tempIconData["Rotation"] then
            TLx, TLy = RotationPoint(tempIconData["Rotation"]),RotationPoint(tempIconData["Rotation"] + 90)
            BLx, BLy = RotationPoint(tempIconData["Rotation"] + 90),RotationPoint(tempIconData["Rotation"] + 180)
            TRx, TRy = RotationPoint(tempIconData["Rotation"] + 270),RotationPoint(tempIconData["Rotation"])
            BRx, BRy = RotationPoint(tempIconData["Rotation"] + 180),RotationPoint(tempIconData["Rotation"] + 270)
            if type(tempIconData["Zoom"]) == "number" then 
                TLx, TLy = RotationZoom(TLx, TLy, tempIconData["Zoom"])
                BLx, BLy = RotationZoom(BLx, BLy, tempIconData["Zoom"])
                TRx, TRy = RotationZoom(TRx, TRy, tempIconData["Zoom"])
                BRx, BRy = RotationZoom(BRx, BRy, tempIconData["Zoom"])      
            end
        elseif tempIconData["LeftCoord"] and tempIconData["RightCoord"] and tempIconData["TopCoord"] and tempIconData["BottomCoord"] then 
            TLy = tempIconData["TopCoord"]
            TRy = tempIconData["TopCoord"]
            BLy = tempIconData["BottomCoord"]
            BRy = tempIconData["BottomCoord"]
            TLx = tempIconData["LeftCoord"]
            BLx = tempIconData["LeftCoord"]
            TRx = tempIconData["RightCoord"]
            BRx = tempIconData["RightCoord"]
        end
        if tempIconData["HFlip"] then 
            local tempx,tempy = TRx, TRy
            TRx, TRy = TLx, TLy
            TLx, TLy = tempx, tempy

            tempx,tempy = BRx, BRy
            BRx, BRy = BLx, BLy
            BLx, BLy = tempx, tempy
        end
        self.Icon:SetTexCoord(TLx, TLy, BLx, BLy, TRx, TRy, BRx, BRy)

        if type(tempIconData["Theme"]) == "string" then 
            if tempIconData["Theme"] == "Highlight" then 
                self.self:SetThemedColor(self.Icon,"Highlight")
            end 
        else 
            self.Icon:SetVertexColor(tempIconData["Red"] or 1, tempIconData["Green"] or 1, tempIconData["Blue"] or 1, 1)
        end
    else
        if self.Icon then
            self.Icon:Release()
            self.Icon = nil
        end
    end
end

local function SetSize(self, w, h)
    if w then
		self.InternalWidth = w;
    end
    if h then
		self.InternalHeight = h;
    end
    self:SetSize(self.InternalWidth, self.InternalHeight)
    for i, v in ipairs(self.OnSizeChangedFuncs) do --at time the inbuilt OnSizeChanged event does not correctly trigger
        v(self)
    end
end

local function PresetBorders(self, State, Focus, Borderless)
    State = State or "Normal"
    local SecondaryColor = "Border"
    local SecondaryAlpha = 1
    if Focus then
        SecondaryColor = "Highlight"
    elseif Borderless then 
        SecondaryAlpha = 0
    end
    if State == "Pushed" then
        self:SetThemedBackdrop("Border", "Highlight", nil, SecondaryColor,nil,nil,SecondaryAlpha)
    elseif State == "MouseOver" then
        self:SetThemedBackdrop("Border", "Highlight", 0.5, SecondaryColor,nil,nil,SecondaryAlpha)
    elseif Borderless then 
        self:SetThemedBackdrop("Border", "Border", nil, SecondaryColor,nil,0,SecondaryAlpha)
    else
        self:SetThemedBackdrop("Border", "Border", nil, SecondaryColor,nil,nil,SecondaryAlpha)
    end
end

local function SetThemedBackdrop(self, Type, PrimaryTheme, PrimaryShader, SecondaryTheme, SecondaryShader, PrimaryAlpha, SecondaryAlpha)
    Type = Type or "ALL"
    if Type == "Clear" then 
        if self.BorderL then
            self.BorderL:Release()
            self.BorderL = nil
        end 
        if self.BorderT then
            self.BorderT:Release()
            self.BorderT = nil
        end 
        if self.BorderR then
            self.BorderR:Release()
            self.BorderR = nil
        end 
        if self.BorderB then
            self.BorderB:Release()
            self.BorderB = nil
        end 
        if self.BackdropTexture then
            self.BackdropTexture:Release()
            self.BackdropTexture = nil
        end 
        return nil
    else
        PrimaryShader = PrimaryShader or 1
        SecondaryShader = SecondaryShader or 1
        PrimaryAlpha = PrimaryAlpha or 1
        SecondaryAlpha = SecondaryAlpha or 1
        if Type == "Backdrop" or Type == "ALL" then 
            if not self.BackdropTexture then
                self.BackdropTexture = self.self:FetchTexture(self, "BACKGROUND")
                self.BackdropTexture:SetPoint("BOTTOMLEFT", self.self.BorderSize, self.self.BorderSize)
                self.BackdropTexture:SetPoint("TOPRIGHT", -self.self.BorderSize, -self.self.BorderSize)
            else 
                self.BackdropTexture:ClearTheme()
            end
            self.BackdropTexture:SetColorTexture(1, 1, 1, 1)

            local PrimaryTheme = PrimaryTheme or "Backdrop"
            local SecondaryTheme = SecondaryTheme or PrimaryTheme 

            self.self:SetThemedColor(self.BackdropTexture,PrimaryTheme,SecondaryTheme,PrimaryShader,SecondaryShader, PrimaryAlpha, SecondaryAlpha)
        end
        if Type == "Border" or Type == "ALL" then 
            if not self.BorderL then
                self.BorderL = self.self:FetchTexture(self, "BORDER")
                self.BorderL:SetPoint("TOPLEFT")
                self.BorderL:SetPoint("BOTTOMLEFT")
                self.BorderL:SetWidth(self.self.BorderSize)
            else 
                self.BorderL:ClearTheme()
            end
            self.BorderL:SetColorTexture(1, 1, 1, 1)
    
            if not self.BorderR then
                self.BorderR = self.self:FetchTexture(self, "BORDER")
                self.BorderR:SetPoint("TOPRIGHT")
                self.BorderR:SetPoint("BOTTOMRIGHT")
                self.BorderR:SetWidth(self.self.BorderSize)
            else 
                self.BorderR:ClearTheme()
            end
            self.BorderR:SetColorTexture(1, 1, 1, 1)
    
            if not self.BorderT then
                self.BorderT = self.self:FetchTexture(self, "BORDER")
                self.BorderT:SetPoint("TOPLEFT", self.BorderL, "TOPRIGHT")
                self.BorderT:SetPoint("TOPRIGHT", self.BorderR, "TOPLEFT")
                self.BorderT:SetHeight(self.self.BorderSize)
            else 
                self.BorderT:ClearTheme()
            end
            self.BorderT:SetColorTexture(1, 1, 1, 1)
    
            if not self.BorderB then
                self.BorderB = self.self:FetchTexture(self, "BORDER")
                self.BorderB:SetPoint("BOTTOMLEFT", self.BorderL, "BOTTOMRIGHT")
                self.BorderB:SetPoint("BOTTOMRIGHT", self.BorderR, "BOTTOMLEFT")
                self.BorderB:SetHeight(self.self.BorderSize)
            else 
                self.BorderB:ClearTheme()
            end
            self.BorderB:SetColorTexture(1, 1, 1, 1)

            PrimaryTheme = PrimaryTheme or "Border"
            SecondaryTheme = SecondaryTheme or PrimaryTheme 

            self.self:SetThemedColor(self.BorderL,PrimaryTheme,nil,PrimaryShader,nil, PrimaryAlpha)
            self.self:SetThemedColor(self.BorderR,SecondaryTheme,nil,SecondaryShader,nil, SecondaryAlpha)
            self.self:SetThemedColor(self.BorderT,PrimaryTheme,SecondaryTheme,PrimaryShader,SecondaryShader, PrimaryAlpha, SecondaryAlpha)
            self.self:SetThemedColor(self.BorderB,PrimaryTheme,SecondaryTheme,PrimaryShader,SecondaryShader, PrimaryAlpha, SecondaryAlpha)
        end 
    end 
end

local function SetColorBackdrop(self, Type, r, g, b, a, r2, g2, b2, a2)
    Type = Type or "ALL"
    if Type == "Clear" then 
        if self.BorderL then
            self.BorderL:Release()
            self.BorderL = nil
        end 
        if self.BorderT then
            self.BorderT:Release()
            self.BorderT = nil
        end 
        if self.BorderR then
            self.BorderR:Release()
            self.BorderR = nil
        end 
        if self.BorderB then
            self.BorderB:Release()
            self.BorderB = nil
        end 
        if self.BackdropTexture then
            self.BackdropTexture:Release()
            self.BackdropTexture = nil
        end 
        self.FixedBorderColor = nil
        return nil
    end
    if Type == "ALL" or Type == "Backdrop" or Type == "FULL_Backdrop" then
        if not self.BackdropTexture then
            self.BackdropTexture = self.self:FetchTexture(self, "BACKGROUND")
            self.BackdropTexture:SetPoint("BOTTOMLEFT", self.self.BorderSize, self.self.BorderSize)
            self.BackdropTexture:SetPoint("TOPRIGHT", -self.self.BorderSize, -self.self.BorderSize)
        else 
            self.BackdropTexture:ClearTheme()
        end
        self.BackdropTexture:SetColorTexture(1, 1, 1, 1)

        local r = r or self.self.BackdropColor.r
        local b = b or self.self.BackdropColor.b
        local g = g or self.self.BackdropColor.g
        local a = a or self.self.BackdropColor.a
        local r2 = r2 or r
        local b2 = b2 or b
        local g2 = g2 or g
        local a2 = a2 or a

        self.BackdropTexture:SetGradientAlpha("HORIZONTAL", r, g, b, a, r2, g2, b2, a2)
    end
    if Type == "ALL" or Type == "Border" or (Type == "FULL_Backdrop" and not self.FixedBorderColor) then
        if Type == "Border" then 
            self.FixedBorderColor = true 
        else 
            self.FixedBorderColor = nil
        end 
        if not self.BorderL then
            self.BorderL = self.self:FetchTexture(self, "BORDER")
            self.BorderL:SetPoint("TOPLEFT")
            self.BorderL:SetPoint("BOTTOMLEFT")
            self.BorderL:SetWidth(self.self.BorderSize)
        else 
            self.BorderL:ClearTheme()
        end
        self.BorderL:SetColorTexture(1, 1, 1, 1)

        if not self.BorderR then
            self.BorderR = self.self:FetchTexture(self, "BORDER")
            self.BorderR:SetPoint("TOPRIGHT")
            self.BorderR:SetPoint("BOTTOMRIGHT")
            self.BorderR:SetWidth(self.self.BorderSize)
        else 
            self.BorderR:ClearTheme()
        end
        self.BorderR:SetColorTexture(1, 1, 1, 1)

        if not self.BorderT then
            self.BorderT = self.self:FetchTexture(self, "BORDER")
            self.BorderT:SetPoint("TOPLEFT", self.BorderL, "TOPRIGHT")
            self.BorderT:SetPoint("TOPRIGHT", self.BorderR, "TOPLEFT")
            self.BorderT:SetHeight(self.self.BorderSize)
        else 
            self.BorderT:ClearTheme()
        end
        self.BorderT:SetColorTexture(1, 1, 1, 1)

        if not self.BorderB then
            self.BorderB = self.self:FetchTexture(self, "BORDER")
            self.BorderB:SetPoint("BOTTOMLEFT", self.BorderL, "BOTTOMRIGHT")
            self.BorderB:SetPoint("BOTTOMRIGHT", self.BorderR, "BOTTOMLEFT")
            self.BorderB:SetHeight(self.self.BorderSize)
        else 
            self.BorderB:ClearTheme()
        end
        self.BorderB:SetColorTexture(1, 1, 1, 1)

        r = r or self.self.BorderColor.r
        b = b or self.self.BorderColor.b
        g = g or self.self.BorderColor.g
        a = a or self.self.BorderColor.a
        r2 = r2 or r
        b2 = b2 or b
        g2 = g2 or g
        a2 = a2 or a

        self.BorderL:SetVertexColor(r, g, b, a)
        self.BorderR:SetVertexColor(r2, g2, b2, a2)
        self.BorderT:SetGradientAlpha("HORIZONTAL", r, g, b, a, r2, g2, b2, a2)
        self.BorderB:SetGradientAlpha("HORIZONTAL", r, g, b, a, r2, g2, b2, a2)
    end
end 

local function OnRelease(self, Func)
    insert(self.OnReleaseFuncs, Func)
end

local function SizeChanged(self, Func)
    insert(self.OnSizeChangedFuncs, Func)
end

local function StopMoving(self, Button)
    self:SetScript("OnUpdate", nil)
    if self.MouseInteraction and self.Interactable then
        self:MouseInteraction(true, Button)
    end
    if self.Movable and self.OnMove then
        self.OnMove(self)
    end
end

local function InternalMoveFunc_OnUpdate(self, elapsed)
    if self.CheckCombatLock and InCombatLockdown() then
        return StopMoving(self)
    end
    local MouseX, MouseY = GetCursorPosition()
    local Scale = self:GetEffectiveScale() 
    MouseX = MouseX / Scale
    MouseY = MouseY / Scale
    self:SetPoint(self.PosParameters.Point, self.PosParameters.Ref, self.PosParameters.RefPoint, MouseX + self.PosParameters.xDif, MouseY + self.PosParameters.yDif)
end

local function InternalMoveFunc_OnUpdate_WithFeedback(self, elapsed)
    if self.CheckCombatLock and InCombatLockdown() then
        return StopMoving(self)
    end
    local MouseX, MouseY = GetCursorPosition()
    local Scale = self:GetEffectiveScale()
    MouseX = MouseX / Scale
    MouseY = MouseY / Scale
    self:SetPoint(self.PosParameters.Point, self.PosParameters.Ref, self.PosParameters.RefPoint, MouseX + self.PosParameters.xDif, MouseY + self.PosParameters.yDif)
    self:MouseDownFeedback()
end

local function StartMoving(self, Button)
    if self.MouseInteraction and self.Interactable then
        self:MouseInteraction(false, Button)
    end
    if self.Movable then
        if self.CheckCombatLock and InCombatLockdown() then
            return nil
        end
        local Point, Ref, RefPoint, x, y = self:GetPoint(1)
        x = x or 0
        y = y or 0
        local MouseX, MouseY = GetCursorPosition()
        local Scale = self:GetEffectiveScale()
        self.PosParameters = { Point = Point, Ref = Ref, RefPoint = RefPoint, xDif = x - MouseX / Scale, yDif = y - MouseY / Scale }
        self:ClearAllPoints()
        if self.MouseDownFeedback then
            self:SetScript("OnUpdate", InternalMoveFunc_OnUpdate_WithFeedback)
        else
            self:SetScript("OnUpdate", InternalMoveFunc_OnUpdate)
        end
		if self.OnMoveStart then 
			self.OnMoveStart(self)
		end 
    elseif self.MouseDownFeedback then
        self:SetScript("OnUpdate", function(self) self:MouseDownFeedback() end)
    end
end

local function MakeMovable(self, Movable)
    if Movable and not self.Interactable then
        self:SetScript("OnMouseDown", StartMoving)
        self:SetScript("OnMouseUp", StopMoving)
    elseif not Movable then
        if not self.Interactable then
            self:SetScript("OnMouseDown", nil)
            self:SetScript("OnMouseUp", nil)
            self:EnableMouse(false)
        end
        self:SetScript("OnUpdate", nil)
    end
    self.Movable = Movable;
end

local function SetTooltip(self,TT)
    self.Tooltip = TT
    self:SetScript("OnLeave", function()
        self.self:SetTooltip(self)
        if self.EnterLeave then
            self:EnterLeave();
        end
    end)
    self:SetScript("OnEnter", function()
        if self.Tooltip and self.SpecialTooltip then
            self.self:SetTooltip(self, self.Tooltip, nil, nil, self.SpecialTooltip);
        elseif self.Tooltip then
            self.self:SetTooltip(self, self.Tooltip);
        end
        if self.EnterLeave then
            self:EnterLeave(true);
        end
    end)
end 

local function MakeInteractable(self, Inter)
    if Inter then
        if not self.Movable then
            self:SetScript("OnMouseDown", StartMoving)
            self:SetScript("OnMouseUp", StopMoving)
        end
        self:SetScript("OnLeave", function()
            self.self:SetTooltip(self)
            if self.EnterLeave then
                self:EnterLeave();
            end
        end)
        self:SetScript("OnEnter", function()
            if self.Tooltip and self.SpecialTooltip then
                self.self:SetTooltip(self, self.Tooltip, nil, nil, self.SpecialTooltip);
            elseif self.Tooltip then
                self.self:SetTooltip(self, self.Tooltip);
            end
            if self.EnterLeave then
                self:EnterLeave(true);
            end
        end)
    else
        self:SetScript("OnLeave")
        self:SetScript("OnEnter")
        if not self.Movable then
            self:SetScript("OnMouseDown", nil)
            self:SetScript("OnMouseUp", nil)
            self:EnableMouse(false)
        end
    end
    self.Interactable = Inter;
end

function _Avyr_Global_ADDON_Elements_Lib.WidgetsMixin:CreateFrame(Parent)
    local Frame = 0
    for i, v in ipairs(Frames) do
        if v.Released then
            Frame = i
            break
        end
    end
    if Frame == 0 then
        Frame = 1
        insert(Frames, Frame, CreateFrame("Frame", nil, Parent or UIParent))
        Frames[Frame]:SetClampedToScreen(false)

        Frames[Frame].LM = {};		
		Frames[Frame].InternalWidth = 0
		Frames[Frame].InternalHeight = 0
		Frames[Frame].OnSizeChanged = function(self, width, height)
			if width ~= self.InternalWidth then 
				self:SetWidth(self.InternalWidth)
			end 
			if Height ~= self.InternalHeight then 
				self:SetHeight(self.InternalHeight)			
			end
		end
    end
    Frames[Frame].OnReleaseFuncs = {}
    Frames[Frame].OnSizeChangedFuncs = {}

    Frames[Frame].Release = Release
    Frames[Frame].SetColorBackdrop = SetColorBackdrop
    Frames[Frame].SetThemedBackdrop = SetThemedBackdrop
    Frames[Frame].OnRelease = OnRelease
    Frames[Frame].SizeChanged = SizeChanged
    Frames[Frame].SetText = SetText
    Frames[Frame].SetIcon = SetIcon
    Frames[Frame].Size = SetSize
    Frames[Frame].SetDisable = SetDisable
    Frames[Frame].PresetBorders = PresetBorders
    Frames[Frame].MakeMovable = MakeMovable
    Frames[Frame].MakeInteractable = MakeInteractable
    Frames[Frame].SetTooltip = SetTooltip
    Frames[Frame].ClearPopup = ClearPopup

    Frames[Frame].self = self

    Frames[Frame]:SetParent(Parent or UIParent)
    Frames[Frame]:ClearAllPoints()
    Frames[Frame]:Show()
    Frames[Frame].Released = false

    return Frames[Frame]
end
